#!/usr/bin/perl
#
# cdload.perl
#
# this program draws most of the forms associated with encoding new
# CDs into the database (ripbot).
#

########################################################################
# perl script boilerplate.
# set tunebot root and load utility module.
#
BEGIN {
    $tunebot_root="XXX-the-tunebot-install-root";
    @INC=($tunebot_root."/lib", @INC);
}
use Tunebot;
#
# end of perl boilerplate
########################################################################

$cgibinurl=$configs{"CGI_BIN_URL"};

$debug=0;
$ses_directory=$tunebot_root."/data/loader";
$uprefs_directory=$tunebot_root."/data/userprefs";

sub numeric { $a <=> $b; }

#
# add/replace a tag value into the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# creates the session if it doesn't already exist.
#
sub addSesTag {
    open (O, ">>$ses_directory/$usrnam.$sesno");
    print O @_, "\n";
    close(O);
    chmod 0666, "$ses_directory/$usrnam.$sesno";
}

#
# lookup a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns the tag value, or "" if not defined.
#
sub getSesTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

#
# lookup the entire history of a tag value from the current session.
# assumes $ses_directory, $usrnam and $sesno are set.
# returns an array of values, in order found.
#
sub getSesTagAll {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my $val;
    my @lst;
    my $i;
    open (I, "$ses_directory/$usrnam.$sesno");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	    @lst = (@lst, $val);
	}
    }
    close(I);
    return @lst;
}

#
# add/replace a tag value into the user preferences
# assumes $uprefs_directory and $usrnam are set.
# creates the preferences file if it doesn't already exist.
#
sub addUPrefsTag {
    open (O, ">>$uprefs_directory/$usrnam");
    print O @_, "\n";
    close(O);
    chmod 0666, "$uprefs_directory/$usrnam";
}

#
# lookup a tag value from the user preferences.
# assumes $uprefs_directory and $usrnam are set.
# returns the tag value, or "" if not defined.
#
sub getUPrefsTag {
    my ($tag) = $_[0] . "=";
    my ($l) = length($tag);
    my ($val) = $_[1];
    my $i;
    open (I, "$uprefs_directory/$usrnam");
    while ($i = <I>) {
	chomp $i;
	if (substr($i, 0, $l) eq $tag) {
	    $val = $i;
	    $val =~ s/^[^=]*=//;
	}
    }
    close(I);
    return $val;
}

Tunebot::suckInCookies;
$tunebot_user=$cookie{TUNEBOTUSER};
Tunebot::suckInPostGet;

Tunebot::suckInTags();
Tunebot::suckInAutotags();

#
# security hole - we trust these as filenames and stuff.
#
$usrnam=$tunebot_user;
$usrnam =~ tr/[A-Za-z0-9]//dc;
$sesno=$posts{"sesno"};
$sesno =~ tr/[0-9]//dc;

#
# special action - create a new session, then refresh screen
#
if ($usrnam ne "" && $posts{"newses"} ne "") {
    $sesno=0;
    do {
	$sesno++;
	opendir(D, $ses_directory);
	foreach $i (readdir(D)) {
	    if ($i =~ /^$usrnam\.[0-9]+$/) {
		$i =~ s/.*\.//;
		if ($i >= $sesno) {
		    $sesno = $i + 1;
		}
	    }
	}
	closedir(D);
    } while ( -r "$ses_directory/$usrnam.$sesno");
    addSesTag("STATE=EMPTY");
    addSesTag("USRNAM=$usrnam");
    addSesTag("SESSION=$sesno");
    addSesTag("CTIME=".time());
    addSesTag("MESSAGE=New Session Created");
    addSesTag("NAME=unnamed");
    $i = getUPrefsTag("ENCODINGPLAN", "SMALL");
    addSesTag("ENCODINGPLAN=$i");
    $i = getUPrefsTag("MP3RATE", "256");
    addSesTag("MP3RATE=$i");
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - take us back to user's list of sessions.
#
if ($usrnam ne "" && $posts{"seslist"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - delete a session and return to session list
#
if ($usrnam ne "" && $sesno ne "" && ($posts{"delete"} ne "" || $posts{"delete.x"} ne "")) {
    unlink("$ses_directory/$usrnam.$sesno");
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - enable session debugging
#
if ($usrnam ne "" && $sesno ne "" && $posts{"debugses"} ne "") {
    addSesTag("SESSIONDEBUG=".$posts{"debugses"});
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - read table of contents from CD and refresh the screen
#
if ($usrnam ne "" && $sesno ne "" && $posts{"readtoc"} ne "") {
    addSesTag("STATE=READTOC");
    system $tunebot_root."/bin/getdisctoc.pl", $usrnam, $sesno;
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - look up the CD at CDDB and refresh the screen.
#
if ($usrnam ne "" && $sesno ne "" && $posts{"getcddb"} ne "") {
    addSesTag("STATE=GETCDDB");
    # XXX this is a bug - it may block if CDDB is broken any way.
    system $tunebot_root."/bin/cddblookup.pl", $usrnam, $sesno;
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - begin ripping and encoding CD
#
if ($usrnam ne "" && $sesno ne "" && $posts{"encode"} ne "") {
    # XXX sanity check of values
    addSesTag("STATE=ENCODING");
    system $tunebot_root."/bin/cdencodebegin", $usrnam, $sesno;
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - stop ripping and encoding CD
#
if ($usrnam ne "" && $sesno ne "" && $posts{"stopenc"} ne "") {
    # XXX not really can just do this
#    addSesTag("STATE=IDLE");
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdload.cgi?sesno=$sesno\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# special action - switch to tags edit form
#
if ($usrnam ne "" && $sesno ne "" && $posts{"edittags"} ne "") {
    print "Content-type: text/html\n";
    print "\n";
    print "<html>\n";
    print "<head>\n";
    print "<META HTTP-EQUIV=REFRESH CONTENT=\"0; URL=cdedit.cgi?sesno=$sesno&begin=1\">\n";
    print "</head>\n";
    print "</html>\n";
    exit 0;
}

#
# normal (no action) - begin to render the session status display
#
Tunebot::printPageHeader "RipBot";

#
# print RipBot banner and user+session if they're defined.
#
print "<p>\n";
print "<center><font size=+2>" . $configs{"BANNER_STRING"} . " RipBot</font>";
if ($usrnam ne "") {
    print " <font size=+2 color=\"#723a4c\"> [ $usrnam";
    if ($sesno ne "") {
	print "#$sesno";
	print "<em> / ".getSesTag("NAME")."</em> : ".getSesTag("STATE");
    }
    print " ] </font>\n";
}
print "</center>\n";
print "<p>\n";

#
# now we render whichever form is appropriate for the current
# state of the user / session.
#

SWITCH: {

#
# this is for when there's no username.  only option is to log in.
#
    if ($usrnam eq "") {
	print "Please <a href=\"$cgibinurl/login.cgi\">log in</a> to load CDs.\n";
	last SWITCH;
    }

#
# edit ripping-related user preferences
#
    if ($sesno eq "" && $posts{"prefer"} ne "") {
	print "<center>RipBot User Preferences</center>\n";
	print "<table width=80% align=center>\n";
	print "<tr><td colspan=2><hr></td></tr>\n";
	print "<tr><td width=40%><em>CD Drive on Remote Host</em></td>\n";
	print "<td>".getUPrefsTag("CDHOST")."</td></tr>\n";
	print "<tr><td><em>Encoding Plan</em></td>\n";
	print "<td>".getUPrefsTag("ENCODINGPLAN", "SMALL")."</td></tr>\n";
	print "<tr><td><em>MP3 Bitrate</em></td>\n";
	print "<td>".getUPrefsTag("MP3RATE", "256")."</td></tr>\n";
	print "<tr><td><em>Lock Session to Creating Host</em></td>\n";
	print "<td>".getUPrefsTag("LOCKCREATE", "no")."</td></tr>\n";
	print "<tr><td colspan=2><hr></td></tr>\n";
	print "</table>\n";
	print "Return to <a href=\"$cgibinurl/cdload.cgi\">Sessions</a>\n";
	last SWITCH;
    }

#
# this is when there's no session number.
# allow the user to pick an existing one, or create a new one.
#
    if ($sesno eq "") {
	print "<ul>\n";
	opendir(D, $ses_directory);
	@slist = ();
	foreach $i (readdir(D)) {
	    if ($i =~ /^$usrnam\.[0-9]+$/) {
		$i =~ s/.*\.//;
		@slist = (@slist, $i);
	    }
	}
	closedir(D);
	if ($#slist != -1) {
	    print "<li> Click on an existing session to reattach to it:\n";
	    print "<p>\n";
	    print "<table width=80% align=center>\n";
	    print "<tr><th>Session</th>";
	    print "<th>Name</th>";
	    print "<th>State</th>";
	    print "<th>Last Message</th></tr>\n";
	    print "<tr><td colspan=4><hr><hr></td></tr>\n";
	    foreach $sesno (sort numeric @slist) {
		print "<tr>";
		print "<td align=center><a href=\"$cgibinurl/cdload.cgi?sesno=$sesno\"> $sesno </a></td>";
		print "<td><a href=\"$cgibinurl/cdload.cgi?sesno=$sesno\">".getSesTag("NAME")."</a></td>";
		print "<td align=center> ".getSesTag("STATE")." </td>";
		print "<td> ".getSesTag("MESSAGE")." </td>";
		print "</tr>\n";
	    }
	    print "<tr><td colspan=4><hr></td></tr>\n";
	    print "</table>\n";
	    print "<li> Or, click here to ";
	} else {
	    print "<li> You have no sessions currently defined.\n";
	    print "Click here to ";
	}
	print "<a href=\"$cgibinurl/cdload.cgi?newses=1\"> Create a new session </a>\n";
	print "<p>\n";
	print "<li> Click here to adjust your TuneBot <a href=\"$cgibinurl/cdload.cgi?prefer=1\">User Preferences</a> for RipBot\n";
	last SWITCH;
	print "</ul>\n";
    }

#
# now we have a user and session number
# render the appropriate session form
#

#
# special action - add or change the album cover art
#
    $state = getSesTag("STATE");
    if ($state eq "EMPTY") {
	print "<form action=\"$cgibinurl/cdload.cgi\">";
	print "<table><tr><td>";
	print "<input type=hidden name=sesno value=\"$sesno\">\n";
	print "<input type=submit name=readtoc value=\"Read TOC from CD\">\n";
	print "<input type=submit name=delete value=\"Delete Session\">\n";
	print "<input type=submit name=seslist value=\"Back to Session List\">\n";
	print "</td></tr></table>";
	print "</form>\n";
	print "<hr>\n";
	print "<table width=80% align=center>\n";
	print "<tr><td>Messages:</td><td>\n";
	@lst = reverse getSesTagAll("MESSAGE");
	foreach $i (@lst) {
#		    print "<tr><td><font color=white><em>".$i."</em></font></td></tr>\n";
	    print "<br><font color=white><em>".$i."</em></font>\n";
	}
	print "</td></tr>\n";
	print "</table>\n";
	print "<hr>\n";
	last SWITCH;
    }

    {
	print "<table><tr><td>";
	print "<form action=\"$cgibinurl/cdload.cgi\">";
	print "<input type=hidden name=sesno value=\"$sesno\">\n";
	if ($state eq "IDLE") {
	    print "<input type=submit name=readtoc value=\"Read TOC from CD\">";
	    print "<input type=submit name=getcddb value=\"Lookup at CDDB\">";
	    print "<input type=submit name=edittags value=\"Edit Information\">";
	    print "<input type=submit name=encode value=\"Encode\">";
	    print "<input type=submit name=delete value=\"Delete Session\">";
	}
	if ($state eq "ENCODING") {
	    print "<input type=submit name=stopenc value=\"Stop Encoding\">";
	}
	print "<input type=submit name=seslist value=\"Back to Session List\">";
	print "</form>\n";
	print "</td></tr></table>";
#print "<table width=100% border=1><tr><td width=60%>";
print "<table width=100%><tr><td colspan=2><hr></td></tr><tr><td width=60%>";
	$ntracks = getSesTag("NTRACKS");
	$aifile = getSesTag("AIFILE");
	if ($ntracks > 0) {
	    print "<table width=100% align=center>\n";
	    print "<tr><td>\n";
	    print "<table width=62% align=left>\n";
	    print "<tr><td width=25%><em>Artist Name:</em></td><td> ".getSesTag("ARTISTNAME")." </td></tr>\n";
	    print "<tr><td><em>Artist Sort Key:</em></td><td> ".getSesTag("ARTISTSORT")." </td></tr>\n";
	    print "<tr><td><em>- Directory:</em></td><td> ".getSesTag("ARTISTDIRECTORY")." </td></tr>\n";
	    print "<tr><td><em>Album Name:</em></td><td> ".getSesTag("ALBUMNAME")." </td></tr>\n";
	    print "<tr><td><em>- Directory:</em></td><td> ".getSesTag("ALBUMDIRECTORY")." </td></tr>\n";
	    print "<tr><td><em>- Release Date:</em></td><td> ".getSesTag("ALBUMDATE")." </td></tr>\n";
	    print "<tr><td><em>CDDB:</em></td><td> ".getSesTag("CDDB")." </td></tr>\n";
	    print "<tr><td><em>Encoding Plan:</em></td><td> ".getSesTag("ENCODINGPLAN")." </td></tr>\n";
	    print "<tr><td><em>MP3 Bitrate:</em></td><td> ".getSesTag("MP3RATE")." </td></tr>\n";
	    $x = (getSesTag("TOTLENGTH") * (getSesTag("MP3RATE") * 1024 / 8)) / 1048576;
	    $x = int($x + 0.5);
	    print "<tr><td><em>Est'd Disk Space:</em></td><td> $x MB </td></tr>\n";
	    print "<tr><td><em>Tracks:</em></td><td> $ntracks </td></tr>\n";
	    print "</table>\n";
	    print "</td><td align=center>\n";
	    if ($aifile ne "") {
		print "<a href=\"$cgibinurl/cdeditimage.cgi?sesno=$sesno\"><img src=\"$cgibinurl/cdloadimage.cgi?sesno=$sesno\"></a>\n";
	    } else {
		print "Add<br><a href=\"$cgibinurl/cdeditimage.cgi?sesno=$sesno\">Cover Art</a><br>for album\n";
	    }
	    print "</td></tr>\n";
	    print "</table>\n";
	    print "<table width=100% align=center>\n";
	    print "<tr><td colspan=3><hr></td></tr>\n";
	    for ($i = 1; $i <= $ntracks; ++$i) {
		print "<tr><td width=50% align=left><font size=-1> $i. </font>".getSesTag("TRACKNAME$i")." </td>";
		print "<td width=40%><em> ".getSesTag("TRACKARTIST$i")." </em></td>";
		print "<td> ".getSesTag("TRACKTIME$i")." </td></tr>\n";
	    }
#		    print "<tr><td colspan=3><hr></td></tr>\n";
	    print "</table>\n";
	}

print "</td><td>";
#	print "<hr>\n";
#	print "<table width=80% align=center>\n";
#	print "<tr><td>Messages:</td><td>\n";
print "Messages:<br>\n";
	@lst = reverse getSesTagAll("MESSAGE");
	foreach $i (@lst) {
#		    print "<tr><td><font color=white><em>".$i."</em></font></td></tr>\n";
	    print "<br><font color=white><em>".$i."</em></font>\n";
	}
#	print "</td></tr>\n";
#	print "</table>\n";
#	print "<hr>\n";
print "</td></tr><tr><td colspan=2><hr></td></tr></table>";
	last SWITCH;
    }
}

#
# this is for debugging.
# it lists all the session tags at the end of the form.
# XXX this could be switched on a session DEBUG tag
#
if (getSesTag("SESSIONDEBUG") == 1) {
    print "<p>\n";
    print "<center>*</center>\n";
    print "<hr>\n";
    print "<p>\n";
    print "<center>Session Tags</center>\n";
    print "<font color=white>\n";
    print "<ul><pre>\n";
    open(I, "$ses_directory/$usrnam.$sesno");
    while (<I>) {
	print;
    }
    close(I);
    print "</pre></ul>\n";
    print "</font>\n";
    print "<hr>\n";
    print "<font size=-1><em>Disable</em> <a href=\"$cgibinurl/cdload.cgi?sesno=$sesno&debugses=0\">Session Debugging</a></font>\n";
} else {
    if ($sesno ne "") {
	print "<p>\n";
	print "<font size=-1><em>Enable</em> <a href=\"$cgibinurl/cdload.cgi?sesno=$sesno&debugses=1\">Session Debugging</a></font>\n";
    }
}

Tunebot::printPageTrailer;
exit 0

