
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <getopt.h>

typedef struct id3tag_s {
	char		magic[3];
	char		songname[30];
	char		artist[30];
	char		album[30];
	char		year[4];
	char		note[30];
	unsigned char	style;
} id3tag_t;

#define YEAR 0x01

static int quiet = 0;

int
removetags(
    char *file,
    int remove )
{
    int d;
    id3tag_t tags;
    int rc = 1;

    if ((d = open(file, O_RDWR, 0)) == -1) {
	perror(file);

    } else {
	if (lseek(d, -sizeof(tags), SEEK_END) == (off_t)-1) {
	    if (errno == EINVAL) {
		if (!quiet)
		    fprintf(stderr, "%s: no tags, skipping\n", file);
	    } else
		perror(file);

	} else if (read(d, (char *)&tags, sizeof(tags)) != sizeof(tags)) {
	    fprintf(stderr, "%s: failed to read tags\n", file);

	} else if (tags.magic[0] != 'T'
		|| tags.magic[1] != 'A'
		|| tags.magic[2] != 'G') {
	    if (!quiet)
		fprintf(stderr, "%s: no tags, skipping\n", file);
	    rc = 2;

	} else {
	    if (remove & YEAR) {
		tags.year[0] = 0;
		tags.year[1] = 0;
		tags.year[2] = 0;
		tags.year[3] = 0;
	    }

	    if (lseek(d, -sizeof(tags), SEEK_END) == (off_t)-1) {
		perror(file);

	    } else if (write(d, &tags, sizeof(tags)) != sizeof(tags)) {
		fprintf(stderr, "%s: failed to write tags\n", file);

	    } else {
		rc = 0;
	    }
	}

	close(d);
    }
    return rc;
}

int
main(
    int argc,
    char **argv )
{
    int remove = 0;
    int c;
    int i;
    int done = 0, skipd = 0, errs = 0;

    while ((c = getopt(argc, argv, "yq")) != EOF) {
	switch (c) {
	case 'y':
	    remove |= YEAR;
	    break;
	case 'q':
	    ++quiet;
	    break;
	}
    }

    if (argc - optind < 1) {
	fprintf(stderr, "usage:   removeid3tag [ options ] file ...\n");
	fprintf(stderr, "options: -y      remove year\n");
	fprintf(stderr, "options: -q      don't warn about ignored files (without tags)\n");
	exit(0);
    }

    for (i = optind; i < argc; i++) {
	switch (removetags(argv[i], remove)) {
	case 0:
	    ++done;
	    break;
	case 1:
	    ++errs;
	    break;
	case 2:
	    ++skipd;
	    break;
	}
    }

    printf("%d files tagged, %d skipped, %d errors\n", done, skipd, errs);
    exit(0);
}

