
/*
*  httpget.c
*
*  little program used to fetch context via HTTP.
*  uses libghttp to do all the hard stuff.
*
*  2001-7 manchek
*/

#include <stdio.h>
#include <getopt.h>
#include <ghttp.h>

#define LOG(level, fmt, vars...) do { \
    	if (debug >= (level)) \
	    fprintf(stderr, fmt, ##vars); \
    } while (0)

int
main(
    int argc,
    char **argv )
{
    ghttp_request *request = NULL;
    int num_headers;
    char **headers;
    int i;
    int rc;
    int c;
    char *outfile = 0;
    int argerr = 0;
    char *url = 0;
    int debug = 0;
    FILE *ff = stdout;

    while ((c = getopt(argc, argv, "o:v?")) != EOF) {
	switch (c) {
	case 'o':
	    outfile = optarg;
	    break;
	case 'v':
	    ++debug;
	    break;
	case '?':
	    argerr++;
	    break;
	}
    }
    if (argerr || optind != argc - 1)
	goto usage;

    url = argv[optind];

    if (outfile) {
	if (!(ff = fopen(outfile, "w"))) {
	    fprintf(stderr, "%s: can't write output file\n", outfile);
	    exit(1);
	}
    }

    request = ghttp_request_new();

    LOG(3, "set_uri\n");
    ghttp_set_uri(request, url);

    LOG(3, "set_header\n");
    ghttp_set_header(request, http_hdr_Connection, "close");

    LOG(3, "prepare\n");
    ghttp_prepare(request);

    do {
	LOG(3, "process\n");
	rc = ghttp_process(request);
    } while (rc == ghttp_not_done);

    if (rc != ghttp_done) {
	fprintf(stderr, "Error retrieving document\n");
	exit(1);
    }

    LOG(1, "Length %d\n", ghttp_get_body_len(request));

    LOG(3, "get_header_names\n");
    ghttp_get_header_names(request, &headers, &num_headers);
    LOG(1, "%d headers\n", num_headers);
    for (i = 0; i < num_headers; i++) {
	LOG(2, "[%d] %s: %s\n", i, headers[i],
		ghttp_get_header(request, headers[i]));
	free(headers[i]);
    }

    LOG(3, "get_body\n");
    fwrite(ghttp_get_body(request), ghttp_get_body_len(request), 1, ff);
    fclose(ff);

    ghttp_request_destroy(request);

    exit(0);

usage:
    fprintf(stderr, "usage: httpget [-v] [-o output_file] {url}\n");
    exit(1);
}

